@@ -65,7 +65,7 @@ sparseMixedModels <- function(y,
     } else {
       xW <- x %*% W
     }
-    spam::crossprod.spam(W, xW)
+    MatrixProduct(t(W), xW)
   })
   lWtRinvY <- lapply(X = lRinv, FUN = function(x) {
     spam::crossprod.spam(W, x %*% y) })
@@ -92,7 +92,11 @@ sparseMixedModels <- function(y,
   }
   if (is.null(fixedTheta)) {
     ## Fix a penalty theta, if value becomes high.
-    fixedTheta <- rep(FALSE, length = NvarcompTot)
+    if (!is.null(grpTheta)) {
+      fixedTheta <- rep(FALSE, length = max(grpTheta))
+    } else {
+      fixedTheta <- rep(FALSE, length = NvarcompTot)
+    }
   }
   if (!is.null(grpTheta)) {
     nGrp <- max(grpTheta)
